<?php

namespace Drupal\ipma_weather\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use GuzzleHttp\Exception\RequestException;

/**
 * Provides a 'IPMA Weather' Block
 *
 * @Block(
 *   id = "ipma_weather",
 *   admin_label = @Translation("IPMA Weather"),
 * )
 */
class IpmaWeather extends BlockBase implements ContainerFactoryPluginInterface {

    /**
     * {@inheritdoc}
     */	
    public function __construct(array $configuration, $plugin_id, $plugin_definition) {
        parent::__construct($configuration, $plugin_id, $plugin_definition);
    }

    /**
     * {@inheritdoc}
     */	
    public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
        return new static(
          $configuration, $plugin_id, $plugin_definition
        );
    }

    /**
     * {@inheritdoc}
     */	
    public function defaultConfiguration() {
        return [
          'label_display' => 0,
          'weather_city' => 1110600,
          'weather_cache_time' => 2,
          'show_icon' => TRUE,
          'show_maxtemp' => TRUE,
          'show_mintemp' => TRUE,
          'show_description' => TRUE,
          'show_precipitation' => TRUE,
          'show_date' => TRUE,
        ];
    }

    /**
     * {@inheritdoc}
     */	
    public function blockForm($form, FormStateInterface $form_state) {
        $form['weather'] = [
          '#type' => 'fieldset',
          '#title' => $this->t('IPMA weather settings'),
        ];

        $form['weather']['weather_city'] = [
          '#type' => 'select',
          '#options' => self::getWeatherCities(),
          '#title' => $this->t('City'),
          '#required' => TRUE,
          '#default_value' => $this->configuration['weather_city'],
        ];
        
        $form['weather']['weather_cache_time'] = [
          '#type' => 'number',
          '#min' => 0,
          '#title' => $this->t('Weather cache time'),
          '#description' => $this->t('Weather cache time in hours. Zero means no cache.'),
          '#required' => TRUE,
          '#default_value' => $this->configuration['weather_cache_time'],
        ];

        $form['weather']['show'] = [
          '#type' => 'fieldset',
          '#title' => $this->t('Block settings'),
        ];
        
        $form['weather']['show']['show_icon'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Show Icon'),
          '#default_value' => $this->configuration['show_icon'],
        ];
        
        $form['weather']['show']['show_maxtemp'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Show maximum temperature'),
          '#default_value' => $this->configuration['show_maxtemp'],
        ];
        
        $form['weather']['show']['show_mintemp'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Show minimum temperature'),
          '#default_value' => $this->configuration['show_mintemp'],
        ];
        
        $form['weather']['show']['show_description'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Show weather description'),
          '#default_value' => $this->configuration['show_description'],
        ];
        
        $form['weather']['show']['show_precipitation'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Show precipitation probability'),
          '#default_value' => $this->configuration['show_precipitation'],
        ];
        
        $form['weather']['show']['show_date'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Show weather day'),
          '#default_value' => $this->configuration['show_date'],
        ];
           
        return $form;
    }

    /**
     * {@inheritdoc}
     */	
    public function blockSubmit($form, FormStateInterface $form_state) {
        parent::blockSubmit($form, $form_state);
        $block_values = $form_state->getValue('weather');
        $this->configuration['weather_city'] = $block_values['weather_city'];
        $this->configuration['weather_cache_time'] = $block_values['weather_cache_time'];
        $this->configuration['show_icon'] = $block_values['show']['show_icon'];
        $this->configuration['show_maxtemp'] = $block_values['show']['show_maxtemp'];
        $this->configuration['show_mintemp'] = $block_values['show']['show_mintemp'];
        $this->configuration['show_description'] = $block_values['show']['show_description'];
        $this->configuration['show_precipitation'] = $block_values['show']['show_precipitation'];
        $this->configuration['show_date'] = $block_values['show']['show_date'];
                
        Cache::invalidateTags($this->getCacheTags());
    }

    /**
     * {@inheritdoc}
     */	
    public function build() {
        return [
          '#theme' => 'ipma_weather',
          '#city' => self::getWeather(),
          '#attached' => [
            'library' => 'ipma_weather/drupal.weather'
          ],
          '#cache' => [
            'tags' => $this->getCacheTags(),
          ],
        ];
    }

    /**
     * {@inheritdoc}
     */	
    public function getCacheTags() {
        return Cache::mergeTags(
            parent::getCacheTags(), ['ipma_weather:block']
        );
    }

    public function getWeatherCities() {
        $options = [];
        $client = \Drupal::httpClient();
        try {
            $request = $client->get('http://api.ipma.pt/open-data/distrits-islands.json');
            $responseData = $request->getBody()->getContents();
            $data = json_decode($responseData, true);

            if (json_last_error() === 0) {
                foreach ($data['data'] as $city) {
                    $options[$city['globalIdLocal']] = $city['local'];
                }
            }
            else {
                drupal_set_message('Could not fetch weather information', 'error');
            }
        }
        catch (RequestException $e) {
            drupal_set_message('Could not fetch weather information; HTTP code: ' . $e->getCode(), 'error');
        }

        return $options;
    }

    public function getWeather() {
        $cache = \Drupal::cache()->get('ipma_weather_block');
        if ($cache && $this->configuration['weather_cache_time']) {
            $city = $cache->data;
        }
        else {
            $client = \Drupal::httpClient();
            try {
                $request = $client->get('http://api.ipma.pt/open-data/forecast/meteorology/cities/daily/' . $this->configuration['weather_city'] . '.json');
                $responseData = $request->getBody()->getContents();
                $data = json_decode($responseData, true);

                if (json_last_error() === 0) {
                    foreach ($data['data'] as $city_value) {
                        $city = [
                          'icon' => ($this->configuration['show_icon']) ? $city_value['idWeatherType'] : FALSE,
                          'tmax' => ($this->configuration['show_maxtemp']) ? $city_value['tMax'].'º' : FALSE,
                          'tmin' => ($this->configuration['show_mintemp']) ? $city_value['tMin'].'º' : FALSE,
                          'desc' => ($this->configuration['show_description']) ? self::getDesc($city_value['idWeatherType']) : FALSE,
                          'precip' => ($this->configuration['show_precipitation']) ? $this->t('@precip chance of rain', ['@precip' => $city_value['precipitaProb'].'%']) : FALSE,
                          'date' => ($this->configuration['show_date']) ? $city_value['forecastDate'] : FALSE,
                        ];
                        break;
                    }
                }
                else {
                    drupal_set_message('Could not fetch weather information', 'error');
                }
            }
            catch (RequestException $e) {
                drupal_set_message('Could not fetch weather information; HTTP code: ' . $e->getCode(), 'error');
            }

            \Drupal::cache()->set('ipma_weather_block', $city, strtotime('+'. $this->configuration['weather_cache_time'] .' hours'), $this->getCacheTags());
        }

        return $city;
    }    
    
    public function getDesc($id) {
        $cache = \Drupal::cache()->get('ipma_weather_descs');
        if ($cache) {
            $descs = $cache->data;
        }
        else {
            $descs = [];
            $client = \Drupal::httpClient();
            try {
                $request = $client->get('http://api.ipma.pt/open-data/weather-type-classe.json');
                $responseData = $request->getBody()->getContents();
                $data = json_decode($responseData, true);

                if (json_last_error() === 0) {
                    foreach ($data['data'] as $value) {
                        $descs[$value['idWeatherType']] = $this->t($value['descIdWeatherTypeEN']);
                    }
                }
                else {
                    drupal_set_message('Could not fetch weather information', 'error');
                }
            }
            catch (RequestException $e) {
                drupal_set_message('Could not fetch weather information; HTTP code: ' . $e->getCode(), 'error');
            }

            \Drupal::cache()->set('ipma_weather_descs', $descs, strtotime('+24 hours'));
        }

        return !empty($descs[$id]) ? $descs[$id] : FALSE;
    }
    
}
